# -*- coding: utf-8 -*-
#
#
# Copyright © 2018 Dell Inc. or its subsidiaries. All rights reserved.
# Dell, EMC, and other trademarks are trademarks of Dell Inc. or its subsidiaries.
# Other trademarks may be trademarks of their respective owners.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Vaideeswaran Ganesan
#
from omdrivers.enums.iDRAC.BIOS import *
from omsdk.typemgr.ClassType import ClassType
from omsdk.typemgr.ArrayType import ArrayType
from omsdk.typemgr.BuiltinTypes import *
import sys
import logging

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

logger = logging.getLogger(__name__)


class BIOS(ClassType):
    def __init__(self, parent=None, loading_from_scp=False):
        if PY2:
            super(BIOS, self).__init__("Component", None, parent)
        else:
            super().__init__("Component", None, parent)
        self.AcPwrRcvry = EnumTypeField(None, AcPwrRcvryTypes, parent=self)
        self.AcPwrRcvryDelay = EnumTypeField(None, AcPwrRcvryDelayTypes, parent=self)
        self.AcPwrRcvryUserDelay = IntField(None, parent=self)
        self.AddrBasMir = EnumTypeField(None, AddrBasMirTypes, parent=self)
        self.AesNi = StringField("", parent=self)
        self.AssetTag = StringField("", parent=self)
        self.AttemptFastBoot = EnumTypeField(None, AttemptFastBootTypes, parent=self)
        self.AttemptFastBootCold = EnumTypeField(None, AttemptFastBootColdTypes, parent=self)
        self.BatteryStatus = StringField("", parent=self)
        self.BiosBootSeq = StringField("", parent=self, rebootRequired=True)
        self.BiosUpdateControl = EnumTypeField(None, BiosUpdateControlTypes, parent=self)
        self.BootMode = EnumTypeField(None, BootModeTypes, parent=self, rebootRequired=True)
        self.BootSeq = StringField("", parent=self)
        self.BootSeqRetry = EnumTypeField(None, BootSeqRetryTypes, parent=self)
        self.BrowserDebugMode = EnumTypeField(None, BrowserDebugModeTypes, parent=self)
        self.BrowserMode = EnumTypeField(None, BrowserModeTypes, parent=self)
        self.BugChecking = EnumTypeField(None, BugCheckingTypes, parent=self)
        self.CTOMasking = EnumTypeField(None, CTOMaskingTypes, parent=self)
        self.CkeThrottling = EnumTypeField(None, CkeThrottlingTypes, parent=self)
        self.ClpOutput = EnumTypeField(None, ClpOutputTypes, parent=self)
        self.ClusterOnDie = EnumTypeField(None, ClusterOnDieTypes, parent=self)
        self.CollaborativeCpuPerfCtrl = EnumTypeField(None, CollaborativeCpuPerfCtrlTypes, parent=self)
        self.ConTermType = EnumTypeField(None, ConTermTypeTypes, parent=self)
        self.ControlledTurbo = EnumTypeField(None, ControlledTurboTypes, parent=self)
        self.ControlledTurboExtended = EnumTypeField(None, ControlledTurboExtendedTypes, parent=self)
        self.CorePerfBoost = EnumTypeField(None, CorePerfBoostTypes, parent=self)
        self.CorrEccSmi = EnumTypeField(None, CorrEccSmiTypes, parent=self)
        self.CpuInterconnectBusLinkPower = EnumTypeField(None, CpuInterconnectBusLinkPowerTypes, parent=self)
        self.CpuInterconnectBusSpeed = EnumTypeField(None, CpuInterconnectBusSpeedTypes, parent=self)
        self.CurrentEmbVideoState = StringField("", parent=self)
        self.CurrentLimit = IntField(None, parent=self)
        self.CurrentMemOpModeState = StringField("", parent=self)
        self.DataReuse = EnumTypeField(None, DataReuseTypes, parent=self)
        self.DcuIpPrefetcher = EnumTypeField(None, DcuIpPrefetcherTypes, parent=self)
        self.DcuStreamerPrefetcher = EnumTypeField(None, DcuStreamerPrefetcherTypes, parent=self)
        self.DebugErrorLevel = StringField("", parent=self)
        self.DellAutoDiscovery = EnumTypeField(None, DellAutoDiscoveryTypes, parent=self)
        self.DellWyseP25BIOSAccess = EnumTypeField(None, DellWyseP25BIOSAccessTypes, parent=self)
        self.DeviceUnhide = EnumTypeField(None, DeviceUnhideTypes, parent=self)
        self.Dfx = EnumTypeField(None, DfxTypes, parent=self)
        self.DirectMediaInterfaceSpeed = EnumTypeField(None, DirectMediaInterfaceSpeedTypes, parent=self)
        self.DmaVirtualization = EnumTypeField(None, DmaVirtualizationTypes, parent=self)
        self.DynamicCoreAllocation = EnumTypeField(None, DynamicCoreAllocationTypes, parent=self)
        self.EmbNic1 = EnumTypeField(None, EmbNic1Types, parent=self)
        self.EmbNic1Nic2 = EnumTypeField(None, EmbNic1Nic2Types, parent=self)
        self.EmbNic2 = EnumTypeField(None, EmbNic2Types, parent=self)
        self.EmbNic3 = EnumTypeField(None, EmbNic3Types, parent=self)
        self.EmbNic3Nic4 = EnumTypeField(None, EmbNic3Nic4Types, parent=self)
        self.EmbNic4 = EnumTypeField(None, EmbNic4Types, parent=self)
        self.EmbNicPort1BootProto = EnumTypeField(None, EmbNicPort1BootProtoTypes, parent=self)
        self.EmbNicPort2BootProto = EnumTypeField(None, EmbNicPort2BootProtoTypes, parent=self)
        self.EmbNicPort3BootProto = EnumTypeField(None, EmbNicPort3BootProtoTypes, parent=self)
        self.EmbNicPort4BootProto = EnumTypeField(None, EmbNicPort4BootProtoTypes, parent=self)
        self.EmbSata = EnumTypeField(None, EmbSataTypes, parent=self)
        self.EmbSataRSTeDebug = EnumTypeField(None, EmbSataRSTeDebugTypes, parent=self)
        self.EmbSataShadow = EnumTypeField(None, EmbSataShadowTypes, parent=self)
        self.EmbSataTestMode = EnumTypeField(None, EmbSataTestModeTypes, parent=self)
        self.EmbVideo = EnumTypeField(None, EmbVideoTypes, parent=self)
        self.EnergyEfficientTurbo = EnumTypeField(None, EnergyEfficientTurboTypes, parent=self)
        self.EnergyPerformanceBias = EnumTypeField(None, EnergyPerformanceBiasTypes, parent=self)
        self.ErrPrompt = EnumTypeField(None, ErrPromptTypes, parent=self)
        self.ExtSerialConnector = EnumTypeField(None, ExtSerialConnectorTypes, parent=self)
        self.FailSafeBaud = EnumTypeField(None, FailSafeBaudTypes, parent=self)
        self.FanPwrPerf = EnumTypeField(None, FanPwrPerfTypes, parent=self)
        self.ForceInt10 = EnumTypeField(None, ForceInt10Types, parent=self)
        self.FrontLcd = EnumTypeField(None, FrontLcdTypes, parent=self)
        self.GlobalSlotDriverDisable = EnumTypeField(None, GlobalSlotDriverDisableTypes, parent=self)
        self.HddFailover = EnumTypeField(None, HddFailoverTypes, parent=self)
        self.HddSeq = StringField("", parent=self, rebootRequired=True)
        self.HttpDev1EnDis = EnumTypeField(HttpDev1EnDisTypes.Disabled, HttpDev1EnDisTypes, parent=self)
        self.HttpDev1Interface = StringField("", parent=self)
        self.HttpDev1Protocol = EnumTypeField(HttpDev1ProtocolTypes.IPv4, HttpDev1ProtocolTypes, parent=self)
        self.HttpDev1Uri = StringField("", parent=self)
        self.HttpDev1VlanEnDis = EnumTypeField(HttpDev1VlanEnDisTypes.Disabled, HttpDev1VlanEnDisTypes, parent=self)
        self.HttpDev1VlanId = IntField(None, parent=self)
        self.HttpDev1VlanPriority = IntField(None, parent=self)
        self.HttpDev2EnDis = EnumTypeField(HttpDev2EnDisTypes.Disabled, HttpDev2EnDisTypes, parent=self)
        self.HttpDev2Interface = StringField("", parent=self)
        self.HttpDev2Protocol = EnumTypeField(HttpDev2ProtocolTypes.IPv4, HttpDev2ProtocolTypes, parent=self)
        self.HttpDev2Uri = StringField("", parent=self)
        self.HttpDev2VlanEnDis = EnumTypeField(HttpDev2VlanEnDisTypes.Disabled, HttpDev2VlanEnDisTypes, parent=self)
        self.HttpDev2VlanId = IntField(None, parent=self)
        self.HttpDev2VlanPriority = IntField(None, parent=self)
        self.HttpDev3EnDis = EnumTypeField(HttpDev3EnDisTypes.Disabled, HttpDev3EnDisTypes, parent=self)
        self.HttpDev3Interface = StringField("", parent=self)
        self.HttpDev3Protocol = EnumTypeField(HttpDev3ProtocolTypes.IPv4, HttpDev3ProtocolTypes, parent=self)
        self.HttpDev3Uri = StringField("", parent=self)
        self.HttpDev3VlanEnDis = EnumTypeField(HttpDev3VlanEnDisTypes.Disabled, HttpDev3VlanEnDisTypes, parent=self)
        self.HttpDev3VlanId = IntField(None, parent=self)
        self.HttpDev3VlanPriority = IntField(None, parent=self)
        self.HttpDev4EnDis = EnumTypeField(HttpDev4EnDisTypes.Disabled, HttpDev4EnDisTypes, parent=self)
        self.HttpDev4Interface = StringField("", parent=self)
        self.HttpDev4Protocol = EnumTypeField(HttpDev4ProtocolTypes.IPv4, HttpDev4ProtocolTypes, parent=self)
        self.HttpDev4Uri = StringField("", parent=self)
        self.HttpDev4VlanEnDis = EnumTypeField(HttpDev4VlanEnDisTypes.Disabled, HttpDev4VlanEnDisTypes, parent=self)
        self.HttpDev4VlanId = IntField(None, parent=self)
        self.HttpDev4VlanPriority = IntField(None, parent=self)
        self.IdracDebugMode = EnumTypeField(None, IdracDebugModeTypes, parent=self)
        self.IgnoreIdracCrReq = EnumTypeField(None, IgnoreIdracCrReqTypes, parent=self)
        self.IioPcieGlobalSpeed = EnumTypeField(None, IioPcieGlobalSpeedTypes, parent=self)
        self.InBandManageabilityInterface = EnumTypeField(None, InBandManageabilityInterfaceTypes, parent=self)
        self.InSystemCharacterization = EnumTypeField(None, InSystemCharacterizationTypes, parent=self)
        self.IntNic1Port1BootProto = EnumTypeField(None, IntNic1Port1BootProtoTypes, parent=self)
        self.IntNic1Port2BootProto = EnumTypeField(None, IntNic1Port2BootProtoTypes, parent=self)
        self.IntNic1Port3BootProto = EnumTypeField(None, IntNic1Port3BootProtoTypes, parent=self)
        self.IntNic1Port4BootProto = EnumTypeField(None, IntNic1Port4BootProtoTypes, parent=self)
        self.IntegratedNetwork1 = EnumTypeField(None, IntegratedNetwork1Types, parent=self)
        self.IntegratedNetwork2 = EnumTypeField(None, IntegratedNetwork2Types, parent=self)
        self.IntegratedRaid = EnumTypeField(None, IntegratedRaidTypes, parent=self)
        self.IntegratedSas = EnumTypeField(None, IntegratedSasTypes, parent=self)
        self.IntelTestEventIio = EnumTypeField(None, IntelTestEventIioTypes, parent=self)
        self.IntelTxt = EnumTypeField(None, IntelTxtTypes, parent=self)
        self.InteractivePassword24A = EnumTypeField(None, InteractivePassword24ATypes, parent=self)
        self.InternalSdCard = EnumTypeField(None, InternalSdCardTypes, parent=self)
        self.InternalSdCardPresence = EnumTypeField(None, InternalSdCardPresenceTypes, parent=self)
        self.InternalSdCardPrimaryCard = EnumTypeField(None, InternalSdCardPrimaryCardTypes, parent=self)
        self.InternalSdCardRedundancy = EnumTypeField(None, InternalSdCardRedundancyTypes, parent=self)
        self.InternalUsb = EnumTypeField(None, InternalUsbTypes, parent=self)
        self.InternalUsb1 = EnumTypeField(None, InternalUsb1Types, parent=self)
        self.InternalUsb2 = EnumTypeField(None, InternalUsb2Types, parent=self)
        self.IoNonPostedPrefetch = EnumTypeField(None, IoNonPostedPrefetchTypes, parent=self)
        self.IoatEngine = EnumTypeField(None, IoatEngineTypes, parent=self)
        self.IscsiDev1Con1Auth = EnumTypeField(None, IscsiDev1Con1AuthTypes, parent=self)
        self.IscsiDev1Con1ChapName = StringField("", parent=self)
        self.IscsiDev1Con1ChapSecret = StringField("", parent=self)
        self.IscsiDev1Con1ChapType = EnumTypeField(None, IscsiDev1Con1ChapTypeTypes, parent=self)
        self.IscsiDev1Con1DhcpEnDis = EnumTypeField(None, IscsiDev1Con1DhcpEnDisTypes, parent=self)
        self.IscsiDev1Con1EnDis = EnumTypeField(None, IscsiDev1Con1EnDisTypes, parent=self)
        self.IscsiDev1Con1Gateway = StringField("", parent=self)
        self.IscsiDev1Con1Interface = StringField("", parent=self)
        self.IscsiDev1Con1Ip = StringField("", parent=self)
        self.IscsiDev1Con1IsId = StringField("", parent=self)
        self.IscsiDev1Con1Lun = IntField(None, parent=self)
        self.IscsiDev1Con1Mask = StringField("", parent=self)
        self.IscsiDev1Con1Port = IntField(None, parent=self)
        self.IscsiDev1Con1Protocol = EnumTypeField(None, IscsiDev1Con1ProtocolTypes, parent=self)
        self.IscsiDev1Con1Retry = IntField(None, parent=self)
        self.IscsiDev1Con1RevChapName = StringField("", parent=self)
        self.IscsiDev1Con1RevChapSecret = StringField("", parent=self)
        self.IscsiDev1Con1TargetIp = StringField("", parent=self)
        self.IscsiDev1Con1TargetName = StringField("", parent=self)
        self.IscsiDev1Con1TgtDhcpEnDis = EnumTypeField(None, IscsiDev1Con1TgtDhcpEnDisTypes, parent=self)
        self.IscsiDev1Con1Timeout = IntField(None, parent=self)
        self.IscsiDev1Con1VlanEnDis = EnumTypeField(None, IscsiDev1Con1VlanEnDisTypes, parent=self)
        self.IscsiDev1Con1VlanId = IntField(None, parent=self)
        self.IscsiDev1Con1VlanPriority = IntField(None, parent=self)
        self.IscsiDev1Con2Auth = EnumTypeField(None, IscsiDev1Con2AuthTypes, parent=self)
        self.IscsiDev1Con2ChapName = StringField("", parent=self)
        self.IscsiDev1Con2ChapSecret = StringField("", parent=self)
        self.IscsiDev1Con2ChapType = EnumTypeField(None, IscsiDev1Con2ChapTypeTypes, parent=self)
        self.IscsiDev1Con2DhcpEnDis = EnumTypeField(None, IscsiDev1Con2DhcpEnDisTypes, parent=self)
        self.IscsiDev1Con2EnDis = EnumTypeField(None, IscsiDev1Con2EnDisTypes, parent=self)
        self.IscsiDev1Con2Gateway = StringField("", parent=self)
        self.IscsiDev1Con2Interface = StringField("", parent=self)
        self.IscsiDev1Con2Ip = StringField("", parent=self)
        self.IscsiDev1Con2IsId = StringField("", parent=self)
        self.IscsiDev1Con2Lun = IntField(None, parent=self)
        self.IscsiDev1Con2Mask = StringField("", parent=self)
        self.IscsiDev1Con2Port = IntField(None, parent=self)
        self.IscsiDev1Con2Protocol = EnumTypeField(None, IscsiDev1Con2ProtocolTypes, parent=self)
        self.IscsiDev1Con2Retry = IntField(None, parent=self)
        self.IscsiDev1Con2RevChapName = StringField("", parent=self)
        self.IscsiDev1Con2RevChapSecret = StringField("", parent=self)
        self.IscsiDev1Con2TargetIp = StringField("", parent=self)
        self.IscsiDev1Con2TargetName = StringField("", parent=self)
        self.IscsiDev1Con2TgtDhcpEnDis = EnumTypeField(None, IscsiDev1Con2TgtDhcpEnDisTypes, parent=self)
        self.IscsiDev1Con2Timeout = IntField(None, parent=self)
        self.IscsiDev1Con2VlanEnDis = EnumTypeField(None, IscsiDev1Con2VlanEnDisTypes, parent=self)
        self.IscsiDev1Con2VlanId = IntField(None, parent=self)
        self.IscsiDev1Con2VlanPriority = IntField(None, parent=self)
        self.IscsiDev1ConOrder = StringField("", parent=self)
        self.IscsiDev1EnDis = EnumTypeField(None, IscsiDev1EnDisTypes, parent=self)
        self.IscsiInitiatorName = StringField("", parent=self)
        self.JunoPmEnable = EnumTypeField(None, JunoPmEnableTypes, parent=self)
        self.L1Prefetcher = StringField("", parent=self)
        self.L2Prefetcher = StringField("", parent=self)
        self.LinkDowntrainReporting = EnumTypeField(None, LinkDowntrainReportingTypes, parent=self)
        self.LogicalProc = EnumTypeField(None, LogicalProcTypes, parent=self)
        self.MRCSerialDbgOut = EnumTypeField(None, MRCSerialDbgOutTypes, parent=self)
        self.MeFailureRecoveryEnable = EnumTypeField(None, MeFailureRecoveryEnableTypes, parent=self)
        self.MeUmaEnable = EnumTypeField(None, MeUmaEnableTypes, parent=self)
        self.MemDynamicPwr = EnumTypeField(None, MemDynamicPwrTypes, parent=self)
        self.MemFrequency = EnumTypeField(None, MemFrequencyTypes, parent=self)
        self.MemHotThrottlingMode = EnumTypeField(None, MemHotThrottlingModeTypes, parent=self)
        self.MemLowPower = EnumTypeField(None, MemLowPowerTypes, parent=self)
        self.MemOpMode = EnumTypeField(None, MemOpModeTypes, parent=self)
        self.MemOpVoltage = EnumTypeField(None, MemOpVoltageTypes, parent=self)
        self.MemOptimizer = EnumTypeField(None, MemOptimizerTypes, parent=self)
        self.MemPatrolScrub = EnumTypeField(None, MemPatrolScrubTypes, parent=self)
        self.MemPwrMgmt = EnumTypeField(None, MemPwrMgmtTypes, parent=self)
        self.MemPwrPerf = EnumTypeField(None, MemPwrPerfTypes, parent=self)
        self.MemRefreshRate = EnumTypeField(None, MemRefreshRateTypes, parent=self)
        self.MemTest = EnumTypeField(None, MemTestTypes, parent=self)
        self.MemTestOnFastBoot = EnumTypeField(None, MemTestOnFastBootTypes, parent=self)
        self.MemTestType = EnumTypeField(None, MemTestTypeTypes, parent=self)
        self.MemThrottlingMode = EnumTypeField(MemThrottlingModeTypes.Cltt, MemThrottlingModeTypes, parent=self)
        self.MemVolt = EnumTypeField(None, MemVoltTypes, parent=self)
        self.MemoryFastBootCold = EnumTypeField(None, MemoryFastBootColdTypes, parent=self)
        self.MemoryMappedIOH = EnumTypeField(MemoryMappedIOHTypes.Disabled, MemoryMappedIOHTypes, parent=self)
        self.MemoryMultiThread = EnumTypeField(None, MemoryMultiThreadTypes, parent=self)
        self.MemoryPerBitMargin = EnumTypeField(None, MemoryPerBitMarginTypes, parent=self)
        self.MemoryRmt = EnumTypeField(None, MemoryRmtTypes, parent=self)
        self.MemoryThrottlingMode = EnumTypeField(None, MemoryThrottlingModeTypes, parent=self)
        self.MltRnkSpr = EnumTypeField(None, MltRnkSprTypes, parent=self)
        self.MmioAbove4Gb = EnumTypeField(None, MmioAbove4GbTypes, parent=self)
        self.MonitorMwait = EnumTypeField(None, MonitorMwaitTypes, parent=self)
        self.MultiThreaded = EnumTypeField(None, MultiThreadedTypes, parent=self)
        self.Ndc1PcieLink1 = EnumTypeField(None, Ndc1PcieLink1Types, parent=self)
        self.Ndc1PcieLink2 = EnumTypeField(None, Ndc1PcieLink2Types, parent=self)
        self.Ndc1PcieLink3 = EnumTypeField(None, Ndc1PcieLink3Types, parent=self)
        self.NdcConfigurationSpeed = EnumTypeField(None, NdcConfigurationSpeedTypes, parent=self)
        # readonly attribute
        self.NewSetupPassword = StringField("", parent=self, modifyAllowed=False, deleteAllowed=False)
        # readonly attribute
        self.NewSysPassword = StringField("", parent=self, modifyAllowed=False, deleteAllowed=False)
        self.NmiButton = EnumTypeField(None, NmiButtonTypes, parent=self)
        self.NodeInterleave = EnumTypeField(None, NodeInterleaveTypes, parent=self)
        self.NumLock = EnumTypeField(None, NumLockTypes, parent=self)
        self.NvdimmFactoryDefault = EnumTypeField(None, NvdimmFactoryDefaultTypes, parent=self)
        self.NvdimmFactoryDefault0 = EnumTypeField(None, NvdimmFactoryDefault0Types, parent=self)
        self.NvdimmFactoryDefault1 = EnumTypeField(None, NvdimmFactoryDefault1Types, parent=self)
        self.NvdimmFactoryDefault10 = EnumTypeField(None, NvdimmFactoryDefault10Types, parent=self)
        self.NvdimmFactoryDefault11 = EnumTypeField(None, NvdimmFactoryDefault11Types, parent=self)
        self.NvdimmFactoryDefault2 = EnumTypeField(None, NvdimmFactoryDefault2Types, parent=self)
        self.NvdimmFactoryDefault3 = EnumTypeField(None, NvdimmFactoryDefault3Types, parent=self)
        self.NvdimmFactoryDefault4 = EnumTypeField(None, NvdimmFactoryDefault4Types, parent=self)
        self.NvdimmFactoryDefault5 = EnumTypeField(None, NvdimmFactoryDefault5Types, parent=self)
        self.NvdimmFactoryDefault6 = EnumTypeField(None, NvdimmFactoryDefault6Types, parent=self)
        self.NvdimmFactoryDefault7 = EnumTypeField(None, NvdimmFactoryDefault7Types, parent=self)
        self.NvdimmFactoryDefault8 = EnumTypeField(None, NvdimmFactoryDefault8Types, parent=self)
        self.NvdimmFactoryDefault9 = EnumTypeField(None, NvdimmFactoryDefault9Types, parent=self)
        self.NvdimmFirmwareVer0 = StringField("", parent=self)
        self.NvdimmFirmwareVer1 = StringField("", parent=self)
        self.NvdimmFirmwareVer10 = StringField("", parent=self)
        self.NvdimmFirmwareVer11 = StringField("", parent=self)
        self.NvdimmFirmwareVer2 = StringField("", parent=self)
        self.NvdimmFirmwareVer3 = StringField("", parent=self)
        self.NvdimmFirmwareVer4 = StringField("", parent=self)
        self.NvdimmFirmwareVer5 = StringField("", parent=self)
        self.NvdimmFirmwareVer6 = StringField("", parent=self)
        self.NvdimmFirmwareVer7 = StringField("", parent=self)
        self.NvdimmFirmwareVer8 = StringField("", parent=self)
        self.NvdimmFirmwareVer9 = StringField("", parent=self)
        self.NvdimmFreq0 = StringField("", parent=self)
        self.NvdimmFreq1 = StringField("", parent=self)
        self.NvdimmFreq10 = StringField("", parent=self)
        self.NvdimmFreq11 = StringField("", parent=self)
        self.NvdimmFreq2 = StringField("", parent=self)
        self.NvdimmFreq3 = StringField("", parent=self)
        self.NvdimmFreq4 = StringField("", parent=self)
        self.NvdimmFreq5 = StringField("", parent=self)
        self.NvdimmFreq6 = StringField("", parent=self)
        self.NvdimmFreq7 = StringField("", parent=self)
        self.NvdimmFreq8 = StringField("", parent=self)
        self.NvdimmFreq9 = StringField("", parent=self)
        self.NvdimmInterleaveSupport = EnumTypeField(None, NvdimmInterleaveSupportTypes, parent=self)
        self.NvdimmLocation0 = StringField("", parent=self)
        self.NvdimmLocation1 = StringField("", parent=self)
        self.NvdimmLocation10 = StringField("", parent=self)
        self.NvdimmLocation11 = StringField("", parent=self)
        self.NvdimmLocation2 = StringField("", parent=self)
        self.NvdimmLocation3 = StringField("", parent=self)
        self.NvdimmLocation4 = StringField("", parent=self)
        self.NvdimmLocation5 = StringField("", parent=self)
        self.NvdimmLocation6 = StringField("", parent=self)
        self.NvdimmLocation7 = StringField("", parent=self)
        self.NvdimmLocation8 = StringField("", parent=self)
        self.NvdimmLocation9 = StringField("", parent=self)
        self.NvdimmReadOnly = EnumTypeField(None, NvdimmReadOnlyTypes, parent=self)
        self.NvdimmSerialNum0 = StringField("", parent=self)
        self.NvdimmSerialNum1 = StringField("", parent=self)
        self.NvdimmSerialNum10 = StringField("", parent=self)
        self.NvdimmSerialNum11 = StringField("", parent=self)
        self.NvdimmSerialNum2 = StringField("", parent=self)
        self.NvdimmSerialNum3 = StringField("", parent=self)
        self.NvdimmSerialNum4 = StringField("", parent=self)
        self.NvdimmSerialNum5 = StringField("", parent=self)
        self.NvdimmSerialNum6 = StringField("", parent=self)
        self.NvdimmSerialNum7 = StringField("", parent=self)
        self.NvdimmSerialNum8 = StringField("", parent=self)
        self.NvdimmSerialNum9 = StringField("", parent=self)
        self.NvdimmSize0 = StringField("", parent=self)
        self.NvdimmSize1 = StringField("", parent=self)
        self.NvdimmSize10 = StringField("", parent=self)
        self.NvdimmSize11 = StringField("", parent=self)
        self.NvdimmSize2 = StringField("", parent=self)
        self.NvdimmSize3 = StringField("", parent=self)
        self.NvdimmSize4 = StringField("", parent=self)
        self.NvdimmSize5 = StringField("", parent=self)
        self.NvdimmSize6 = StringField("", parent=self)
        self.NvdimmSize7 = StringField("", parent=self)
        self.NvdimmSize8 = StringField("", parent=self)
        self.NvdimmSize9 = StringField("", parent=self)
        self.NvmeMode = EnumTypeField(None, NvmeModeTypes, parent=self)
        # readonly attribute
        self.OldSetupPassword = StringField("", parent=self, modifyAllowed=False, deleteAllowed=False)
        # readonly attribute
        self.OldSysPassword = StringField("", parent=self, modifyAllowed=False, deleteAllowed=False)
        self.OneTimeBiosBootSeq = StringField("", parent=self)
        self.OneTimeBootMode = EnumTypeField(None, OneTimeBootModeTypes, parent=self)
        self.OneTimeBootModeSeq = StringField("", parent=self)
        self.OneTimeBootSeqDev = EnumTypeField(None, OneTimeBootSeqDevTypes, parent=self)
        self.OneTimeCustomBootStr = StringField("", parent=self)
        self.OneTimeHddSeq = StringField("", parent=self)
        self.OneTimeHddSeqDev = EnumTypeField(None, OneTimeHddSeqDevTypes, parent=self)
        self.OneTimeUefiBootPath = StringField("", parent=self)
        self.OneTimeUefiBootSeq = StringField("", parent=self)
        self.OneTimeUefiBootSeqDev = EnumTypeField(None, OneTimeUefiBootSeqDevTypes, parent=self)
        self.OppSrefEn = EnumTypeField(None, OppSrefEnTypes, parent=self)
        self.OsWatchdogTimer = EnumTypeField(None, OsWatchdogTimerTypes, parent=self)
        self.PCIeErrorInjection = EnumTypeField(None, PCIeErrorInjectionTypes, parent=self)
        self.PCIeLiveErrorRecovery = EnumTypeField(None, PCIeLiveErrorRecoveryTypes, parent=self)
        self.PPRErrInjectionTest = EnumTypeField(None, PPRErrInjectionTestTypes, parent=self)
        self.PasswordStatus = EnumTypeField(None, PasswordStatusTypes, parent=self)
        self.PcieAspmL1 = EnumTypeField(None, PcieAspmL1Types, parent=self)
        self.PerfMonitorDevices = EnumTypeField(None, PerfMonitorDevicesTypes, parent=self)
        self.PersistentMemoryMode = EnumTypeField(None, PersistentMemoryModeTypes, parent=self)
        self.PersistentMemoryScrubbing = EnumTypeField(None, PersistentMemoryScrubbingTypes, parent=self)
        self.PostPackageRepair = EnumTypeField(None, PostPackageRepairTypes, parent=self)
        self.PowerCycleRequest = EnumTypeField(None, PowerCycleRequestTypes, parent=self)
        self.PowerDelivery = EnumTypeField(None, PowerDeliveryTypes, parent=self)
        self.PowerMgmt = EnumTypeField(None, PowerMgmtTypes, parent=self)
        self.PowerSaver = EnumTypeField(None, PowerSaverTypes, parent=self)
        self.Proc1Brand = StringField("", parent=self)
        self.Proc1ControlledTurbo = EnumTypeField(None, Proc1ControlledTurboTypes, parent=self)
        self.Proc1Cores = EnumTypeField(None, Proc1CoresTypes, parent=self)
        self.Proc1Id = StringField("", parent=self)
        self.Proc1L2Cache = StringField("", parent=self)
        self.Proc1L3Cache = StringField("", parent=self)
        self.Proc1Microcode = IntField(None, parent=self)
        self.Proc1NumCores = IntField(None, parent=self)
        self.Proc1TurboCoreNum = EnumTypeField(None, Proc1TurboCoreNumTypes, parent=self)
        self.Proc2Brand = StringField("", parent=self)
        self.Proc2ControlledTurbo = EnumTypeField(None, Proc2ControlledTurboTypes, parent=self)
        self.Proc2Cores = EnumTypeField(None, Proc2CoresTypes, parent=self)
        self.Proc2Id = StringField("", parent=self)
        self.Proc2L2Cache = StringField("", parent=self)
        self.Proc2L3Cache = StringField("", parent=self)
        self.Proc2Microcode = IntField(None, parent=self)
        self.Proc2NumCores = IntField(None, parent=self)
        self.Proc2TurboCoreNum = EnumTypeField(None, Proc2TurboCoreNumTypes, parent=self)
        self.Proc3Brand = StringField("", parent=self)
        self.Proc3ControlledTurbo = EnumTypeField(None, Proc3ControlledTurboTypes, parent=self)
        self.Proc3Cores = EnumTypeField(None, Proc3CoresTypes, parent=self)
        self.Proc3Id = StringField("", parent=self)
        self.Proc3L2Cache = StringField("", parent=self)
        self.Proc3L3Cache = StringField("", parent=self)
        self.Proc3Microcode = IntField(None, parent=self)
        self.Proc3NumCores = IntField(None, parent=self)
        self.Proc3TurboCoreNum = EnumTypeField(None, Proc3TurboCoreNumTypes, parent=self)
        self.Proc4Brand = StringField("", parent=self)
        self.Proc4ControlledTurbo = EnumTypeField(None, Proc4ControlledTurboTypes, parent=self)
        self.Proc4Cores = EnumTypeField(None, Proc4CoresTypes, parent=self)
        self.Proc4Id = StringField("", parent=self)
        self.Proc4L2Cache = StringField("", parent=self)
        self.Proc4L3Cache = StringField("", parent=self)
        self.Proc4Microcode = IntField(None, parent=self)
        self.Proc4NumCores = IntField(None, parent=self)
        self.Proc4TurboCoreNum = EnumTypeField(None, Proc4TurboCoreNumTypes, parent=self)
        self.Proc64bit = StringField("", parent=self)
        self.ProcATS = EnumTypeField(None, ProcATSTypes, parent=self)
        self.ProcAdjCacheLine = EnumTypeField(None, ProcAdjCacheLineTypes, parent=self)
        self.ProcBusSpeed = StringField("", parent=self)
        self.ProcC1E = EnumTypeField(None, ProcC1ETypes, parent=self)
        self.ProcCStates = EnumTypeField(None, ProcCStatesTypes, parent=self)
        self.ProcConfigTdp = EnumTypeField(None, ProcConfigTdpTypes, parent=self)
        self.ProcCoreSpeed = StringField("", parent=self)
        self.ProcCores = EnumTypeField(None, ProcCoresTypes, parent=self)
        self.ProcDpatProDebug = EnumTypeField(None, ProcDpatProDebugTypes, parent=self)
        self.ProcDramPrefetcher = EnumTypeField(None, ProcDramPrefetcherTypes, parent=self)
        self.ProcEmbMemCacheSize = StringField("", parent=self)
        self.ProcEmbMemMode = EnumTypeField(ProcEmbMemModeTypes.Cache, ProcEmbMemModeTypes, parent=self)
        self.ProcEmbMemSystemSize = StringField("", parent=self)
        self.ProcEmbMemTotalSize = StringField("", parent=self)
        self.ProcExecuteDisable = EnumTypeField(None, ProcExecuteDisableTypes, parent=self)
        self.ProcHpcMode = EnumTypeField(None, ProcHpcModeTypes, parent=self)
        self.ProcHtAssist = EnumTypeField(None, ProcHtAssistTypes, parent=self)
        self.ProcHwPrefetcher = EnumTypeField(None, ProcHwPrefetcherTypes, parent=self)
        self.ProcHyperTransport = EnumTypeField(None, ProcHyperTransportTypes, parent=self)
        self.ProcMtrrPatDebug = EnumTypeField(None, ProcMtrrPatDebugTypes, parent=self)
        self.ProcPwrPerf = EnumTypeField(None, ProcPwrPerfTypes, parent=self)
        self.ProcSoftwarePrefetcher = EnumTypeField(None, ProcSoftwarePrefetcherTypes, parent=self)
        self.ProcTurboMode = EnumTypeField(None, ProcTurboModeTypes, parent=self)
        self.ProcVirtualization = EnumTypeField(None, ProcVirtualizationTypes, parent=self)
        self.ProcX2Apic = EnumTypeField(None, ProcX2ApicTypes, parent=self)
        self.PwrButton = EnumTypeField(None, PwrButtonTypes, parent=self)
        self.PxeDev1EnDis = EnumTypeField(None, PxeDev1EnDisTypes, parent=self)
        self.PxeDev1Interface = StringField("", parent=self)
        self.PxeDev1Protocol = EnumTypeField(None, PxeDev1ProtocolTypes, parent=self)
        self.PxeDev1VlanEnDis = EnumTypeField(None, PxeDev1VlanEnDisTypes, parent=self)
        self.PxeDev1VlanId = IntField(None, parent=self)
        self.PxeDev1VlanPriority = IntField(None, parent=self)
        self.PxeDev2EnDis = EnumTypeField(None, PxeDev2EnDisTypes, parent=self)
        self.PxeDev2Interface = StringField("", parent=self)
        self.PxeDev2Protocol = EnumTypeField(None, PxeDev2ProtocolTypes, parent=self)
        self.PxeDev2VlanEnDis = EnumTypeField(None, PxeDev2VlanEnDisTypes, parent=self)
        self.PxeDev2VlanId = IntField(None, parent=self)
        self.PxeDev2VlanPriority = IntField(None, parent=self)
        self.PxeDev3EnDis = EnumTypeField(None, PxeDev3EnDisTypes, parent=self)
        self.PxeDev3Interface = StringField("", parent=self)
        self.PxeDev3Protocol = EnumTypeField(None, PxeDev3ProtocolTypes, parent=self)
        self.PxeDev3VlanEnDis = EnumTypeField(None, PxeDev3VlanEnDisTypes, parent=self)
        self.PxeDev3VlanId = IntField(None, parent=self)
        self.PxeDev3VlanPriority = IntField(None, parent=self)
        self.PxeDev4EnDis = EnumTypeField(None, PxeDev4EnDisTypes, parent=self)
        self.PxeDev4Interface = StringField("", parent=self)
        self.PxeDev4Protocol = EnumTypeField(None, PxeDev4ProtocolTypes, parent=self)
        self.PxeDev4VlanEnDis = EnumTypeField(None, PxeDev4VlanEnDisTypes, parent=self)
        self.PxeDev4VlanId = IntField(None, parent=self)
        self.PxeDev4VlanPriority = IntField(None, parent=self)
        self.QpiBandwidthPriority = EnumTypeField(None, QpiBandwidthPriorityTypes, parent=self)
        self.QpiSpeed = EnumTypeField(None, QpiSpeedTypes, parent=self)
        self.RebootTestCount = IntField(None, parent=self)
        self.RebootTestMode = EnumTypeField(None, RebootTestModeTypes, parent=self)
        self.RebootTestPoint = EnumTypeField(None, RebootTestPointTypes, parent=self)
        self.RedirAfterBoot = EnumTypeField(None, RedirAfterBootTypes, parent=self)
        self.RedundantMem = EnumTypeField(None, RedundantMemTypes, parent=self)
        self.RedundantMemCfgValid = EnumTypeField(None, RedundantMemCfgValidTypes, parent=self)
        self.RedundantMemInUse = EnumTypeField(None, RedundantMemInUseTypes, parent=self)
        self.RedundantOsBoot = EnumTypeField(None, RedundantOsBootTypes, parent=self)
        self.RedundantOsLocation = EnumTypeField(None, RedundantOsLocationTypes, parent=self)
        self.RedundantOsState = EnumTypeField(None, RedundantOsStateTypes, parent=self)
        self.ReportKbdErr = EnumTypeField(None, ReportKbdErrTypes, parent=self)
        self.RipsPresence = EnumTypeField(None, RipsPresenceTypes, parent=self)
        self.RtidSetting = EnumTypeField(None, RtidSettingTypes, parent=self)
        self.S4SupportDebug = EnumTypeField(None, S4SupportDebugTypes, parent=self)
        self.SHA256SetupPassword = StringField("", parent=self)
        self.SHA256SetupPasswordSalt = StringField("", parent=self)
        self.SHA256SystemPassword = StringField("", parent=self)
        self.SHA256SystemPasswordSalt = StringField("", parent=self)
        self.SNC = EnumTypeField(None, SNCTypes, parent=self)
        self.SataPortA = EnumTypeField(None, SataPortATypes, parent=self)
        self.SataPortACapacity = StringField("", parent=self)
        self.SataPortADriveType = StringField("", parent=self)
        self.SataPortAModel = StringField("", parent=self)
        self.SataPortB = EnumTypeField(None, SataPortBTypes, parent=self)
        self.SataPortBCapacity = StringField("", parent=self)
        self.SataPortBDriveType = StringField("", parent=self)
        self.SataPortBModel = StringField("", parent=self)
        self.SataPortC = EnumTypeField(None, SataPortCTypes, parent=self)
        self.SataPortCCapacity = StringField("", parent=self)
        self.SataPortCDriveType = StringField("", parent=self)
        self.SataPortCModel = StringField("", parent=self)
        self.SataPortD = EnumTypeField(None, SataPortDTypes, parent=self)
        self.SataPortDCapacity = StringField("", parent=self)
        self.SataPortDDriveType = StringField("", parent=self)
        self.SataPortDModel = StringField("", parent=self)
        self.SataPortE = EnumTypeField(None, SataPortETypes, parent=self)
        self.SataPortECapacity = StringField("", parent=self)
        self.SataPortEDriveType = StringField("", parent=self)
        self.SataPortEModel = StringField("", parent=self)
        self.SataPortF = EnumTypeField(None, SataPortFTypes, parent=self)
        self.SataPortFCapacity = StringField("", parent=self)
        self.SataPortFDriveType = StringField("", parent=self)
        self.SataPortFModel = StringField("", parent=self)
        self.SataPortG = EnumTypeField(None, SataPortGTypes, parent=self)
        self.SataPortGCapacity = StringField("", parent=self)
        self.SataPortGDriveType = StringField("", parent=self)
        self.SataPortGModel = StringField("", parent=self)
        self.SataPortH = EnumTypeField(None, SataPortHTypes, parent=self)
        self.SataPortHCapacity = StringField("", parent=self)
        self.SataPortHDriveType = StringField("", parent=self)
        self.SataPortHModel = StringField("", parent=self)
        self.SataPortI = EnumTypeField(None, SataPortITypes, parent=self)
        self.SataPortICapacity = EnumTypeField(None, SataPortICapacityTypes, parent=self)
        self.SataPortIDriveType = EnumTypeField(None, SataPortIDriveTypeTypes, parent=self)
        self.SataPortIModel = EnumTypeField(None, SataPortIModelTypes, parent=self)
        self.SataPortJ = EnumTypeField(None, SataPortJTypes, parent=self)
        self.SataPortJCapacity = EnumTypeField(None, SataPortJCapacityTypes, parent=self)
        self.SataPortJDriveType = EnumTypeField(None, SataPortJDriveTypeTypes, parent=self)
        self.SataPortJModel = EnumTypeField(None, SataPortJModelTypes, parent=self)
        self.SataPortK = EnumTypeField(None, SataPortKTypes, parent=self)
        self.SataPortKCapacity = StringField("", parent=self)
        self.SataPortKDriveType = StringField("", parent=self)
        self.SataPortKModel = StringField("", parent=self)
        self.SataPortL = EnumTypeField(None, SataPortLTypes, parent=self)
        self.SataPortLCapacity = StringField("", parent=self)
        self.SataPortLDriveType = EnumTypeField(None, SataPortLDriveTypeTypes, parent=self)
        self.SataPortLModel = StringField("", parent=self)
        self.SataPortM = EnumTypeField(None, SataPortMTypes, parent=self)
        self.SataPortMCapacity = StringField("", parent=self)
        self.SataPortMDriveType = StringField("", parent=self)
        self.SataPortMModel = EnumTypeField(None, SataPortMModelTypes, parent=self)
        self.SataPortN = EnumTypeField(None, SataPortNTypes, parent=self)
        self.SataPortNCapacity = StringField("", parent=self)
        self.SataPortNDriveType = StringField("", parent=self)
        self.SataPortNModel = EnumTypeField(SataPortNModelTypes.SATA_MODEL, SataPortNModelTypes, parent=self)
        self.SccDebugEnabled = EnumTypeField(None, SccDebugEnabledTypes, parent=self)
        self.SecureBoot = EnumTypeField(None, SecureBootTypes, parent=self)
        self.SecureBootMode = EnumTypeField(SecureBootModeTypes.UserMode, SecureBootModeTypes, parent=self)
        self.SecureBootPolicy = EnumTypeField(None, SecureBootPolicyTypes, parent=self)
        self.SecurityFreezeLock = EnumTypeField(None, SecurityFreezeLockTypes, parent=self)
        self.SerialComm = EnumTypeField(None, SerialCommTypes, parent=self)
        self.SerialPortAddress = EnumTypeField(None, SerialPortAddressTypes, parent=self)
        self.SetBootOrderFqdd1 = StringField("", parent=self)
        self.SetBootOrderFqdd10 = StringField("", parent=self)
        self.SetBootOrderFqdd11 = StringField("", parent=self)
        self.SetBootOrderFqdd12 = StringField("", parent=self)
        self.SetBootOrderFqdd13 = StringField("", parent=self)
        self.SetBootOrderFqdd14 = StringField("", parent=self)
        self.SetBootOrderFqdd15 = StringField("", parent=self)
        self.SetBootOrderFqdd16 = StringField("", parent=self)
        self.SetBootOrderFqdd2 = StringField("", parent=self)
        self.SetBootOrderFqdd3 = StringField("", parent=self)
        self.SetBootOrderFqdd4 = StringField("", parent=self)
        self.SetBootOrderFqdd5 = StringField("", parent=self)
        self.SetBootOrderFqdd6 = StringField("", parent=self)
        self.SetBootOrderFqdd7 = StringField("", parent=self)
        self.SetBootOrderFqdd8 = StringField("", parent=self)
        self.SetBootOrderFqdd9 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd1 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd10 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd11 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd12 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd13 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd14 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd15 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd16 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd2 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd3 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd4 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd5 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd6 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd7 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd8 = StringField("", parent=self)
        self.SetLegacyHddOrderFqdd9 = StringField("", parent=self)
        self.SetupPassword = StringField("", parent=self)
        self.SignedFirmwareUpdate = EnumTypeField(None, SignedFirmwareUpdateTypes, parent=self)
        self.Slot1 = EnumTypeField(None, Slot1Types, parent=self)
        self.Slot10 = EnumTypeField(None, Slot10Types, parent=self)
        self.Slot10Bif = EnumTypeField(None, Slot10BifTypes, parent=self)
        self.Slot11 = EnumTypeField(None, Slot11Types, parent=self)
        self.Slot11Bif = EnumTypeField(None, Slot11BifTypes, parent=self)
        self.Slot12 = EnumTypeField(None, Slot12Types, parent=self)
        self.Slot12Bif = EnumTypeField(None, Slot12BifTypes, parent=self)
        self.Slot13 = EnumTypeField(None, Slot13Types, parent=self)
        self.Slot13Bif = EnumTypeField(None, Slot13BifTypes, parent=self)
        self.Slot14Bif = EnumTypeField(None, Slot14BifTypes, parent=self)
        self.Slot1Bif = EnumTypeField(None, Slot1BifTypes, parent=self)
        self.Slot2 = EnumTypeField(None, Slot2Types, parent=self)
        self.Slot2Bif = EnumTypeField(None, Slot2BifTypes, parent=self)
        self.Slot3 = EnumTypeField(None, Slot3Types, parent=self)
        self.Slot3Bif = EnumTypeField(None, Slot3BifTypes, parent=self)
        self.Slot4 = EnumTypeField(None, Slot4Types, parent=self)
        self.Slot4Bif = EnumTypeField(None, Slot4BifTypes, parent=self)
        self.Slot5 = EnumTypeField(None, Slot5Types, parent=self)
        self.Slot5Bif = EnumTypeField(None, Slot5BifTypes, parent=self)
        self.Slot6 = EnumTypeField(None, Slot6Types, parent=self)
        self.Slot6Bif = EnumTypeField(None, Slot6BifTypes, parent=self)
        self.Slot7 = EnumTypeField(None, Slot7Types, parent=self)
        self.Slot7Bif = EnumTypeField(None, Slot7BifTypes, parent=self)
        self.Slot8 = EnumTypeField(None, Slot8Types, parent=self)
        self.Slot8Bif = EnumTypeField(None, Slot8BifTypes, parent=self)
        self.Slot9 = EnumTypeField(None, Slot9Types, parent=self)
        self.Slot9Bif = EnumTypeField(None, Slot9BifTypes, parent=self)
        self.SnoopFilter = EnumTypeField(None, SnoopFilterTypes, parent=self)
        self.SnoopMode = EnumTypeField(None, SnoopModeTypes, parent=self)
        self.SrefProgramming = EnumTypeField(None, SrefProgrammingTypes, parent=self)
        self.SriovGlobalEnable = EnumTypeField(None, SriovGlobalEnableTypes, parent=self)
        self.SubNumaCluster = EnumTypeField(None, SubNumaClusterTypes, parent=self)
        self.SysMemSize = StringField("", parent=self)
        self.SysMemSpeed = StringField("", parent=self)
        self.SysMemType = StringField("", parent=self)
        self.SysMemVolt = StringField("", parent=self)
        self.SysMfrContactInfo = StringField("", parent=self)
        self.SysMgmtNVByte1 = IntField(None, parent=self)
        self.SysMgmtNVByte2 = IntField(None, parent=self)
        self.SysPassword = StringField("", parent=self)
        self.SysProfile = EnumTypeField(None, SysProfileTypes, parent=self)
        self.SystemBiosVersion = StringField("", parent=self)
        self.SystemCpldVersion = StringField("", parent=self)
        self.SystemManufacturer = StringField("", parent=self)
        self.SystemMeVersion = StringField("", parent=self)
        self.SystemMemoryModel = EnumTypeField(SystemMemoryModelTypes.Quadrant, SystemMemoryModelTypes, parent=self)
        self.SystemModelName = StringField("", parent=self)
        self.SystemServiceTag = StringField("", parent=self)
        self.SystemUefiShell = EnumTypeField(None, SystemUefiShellTypes, parent=self)
        self.TXEQWA = EnumTypeField(None, TXEQWATypes, parent=self)
        self.TcmActivation = EnumTypeField(None, TcmActivationTypes, parent=self)
        self.TcmClear = EnumTypeField(None, TcmClearTypes, parent=self)
        self.TcmSecurity = EnumTypeField(None, TcmSecurityTypes, parent=self)
        self.Tpm2Algorithm = EnumTypeField(None, Tpm2AlgorithmTypes, parent=self)
        self.Tpm2Hierarchy = EnumTypeField(None, Tpm2HierarchyTypes, parent=self)
        self.TpmActivation = EnumTypeField(None, TpmActivationTypes, parent=self)
        self.TpmBindingReset = EnumTypeField(None, TpmBindingResetTypes, parent=self)
        self.TpmClear = EnumTypeField(None, TpmClearTypes, parent=self)
        self.TpmCommand = EnumTypeField(None, TpmCommandTypes, parent=self)
        self.TpmFirmware = StringField("", parent=self)
        self.TpmInfo = StringField("", parent=self)
        self.TpmPpiBypassClear = EnumTypeField(None, TpmPpiBypassClearTypes, parent=self)
        self.TpmPpiBypassProvision = EnumTypeField(None, TpmPpiBypassProvisionTypes, parent=self)
        self.TpmSecurity = EnumTypeField(None, TpmSecurityTypes, parent=self)
        self.TpmStatus = StringField("", parent=self)
        self.TraceHubDebug = EnumTypeField(None, TraceHubDebugTypes, parent=self)
        self.UefiBootSeq = StringField("", parent=self)
        self.UefiComplianceVersion = StringField("", parent=self)
        self.UefiPxeIpVersion = EnumTypeField(None, UefiPxeIpVersionTypes, parent=self)
        self.UefiVariableAccess = EnumTypeField(None, UefiVariableAccessTypes, parent=self)
        self.UncoreFrequency = EnumTypeField(None, UncoreFrequencyTypes, parent=self)
        self.UnusedPcieClk = EnumTypeField(None, UnusedPcieClkTypes, parent=self)
        self.Usb3Setting = EnumTypeField(None, Usb3SettingTypes, parent=self)
        self.UsbManagedPort = EnumTypeField(UsbManagedPortTypes.On, UsbManagedPortTypes, parent=self)
        self.UsbPorts = EnumTypeField(None, UsbPortsTypes, parent=self)
        self.UserLcdStr = StringField("", parent=self)
        self.VideoMem = StringField("", parent=self)
        self.WorkloadProfile = EnumTypeField(None, WorkloadProfileTypes, parent=self)
        self.WriteCache = EnumTypeField(None, WriteCacheTypes, parent=self)
        self.WriteDataCrc = EnumTypeField(None, WriteDataCrcTypes, parent=self)
        self.eSataPort1 = EnumTypeField(None, eSataPort1Types, parent=self)
        self.eSataPort1Capacity = StringField("", parent=self)
        self.eSataPort1DriveType = StringField("", parent=self)
        self.eSataPort1Model = StringField("", parent=self)
        self.commit(loading_from_scp)
